// File ExecImageVersion
#include "stdafx.h"
#include "ExecImageVersion.h"

// Construction/Destruction

VersionInfo::VersionInfo()
{
    // m_pszExeName is the real image name in MFC
    CString name = AfxGetApp()->m_pszExeName;
    name += ".EXE";
    InitVer(name);
}

VersionInfo::VersionInfo(LPCTSTR imageName)
{
    InitVer(imageName);
}

VersionInfo::~VersionInfo()
{
    ::free(m_ResourceData);
}

// Attributes

CString VersionInfo::ProductName() const
{
    return GetValue(_T("ProductName"));
}

CString VersionInfo::ProductVersion() const
{
    return GetValue(_T("ProductVersion"));
}

CString VersionInfo::CompanyName() const
{
    return GetValue(_T("CompanyName"));
}

CString VersionInfo::Copyright() const
{
    return GetValue(_T("LegalCopyright"));
}

CString VersionInfo::Comments() const
{
    return GetValue(_T("Comments"));
}

CString VersionInfo::FileDescription() const
{
    return GetValue(_T("FileDescription"));
}

CString VersionInfo::FileVersion() const
{
    return GetValue(_T("FileVersion"));
}

CString VersionInfo::InternalName() const
{
    return GetValue(_T("InternalName"));
}

CString VersionInfo::LegalTrademarks() const
{
    return GetValue(_T("LegalTrademarks"));
}

CString VersionInfo::PrivateBuild() const
{
    return GetValue(_T("PrivateBuild"));
}

CString VersionInfo::SpecialBuild() const
{
    return GetValue(_T("SpecialBuild"));
}

// Construction helper

void VersionInfo::InitVer(LPCTSTR imageName)
{
    DWORD resourceSize, zeroHandle = 0;
    LPTSTR image = const_cast<LPTSTR>(imageName); // WinAPI wants non-const arguments

    m_ResourceData = NULL;
    resourceSize = ::GetFileVersionInfoSize(image, &zeroHandle);
    if (zeroHandle == 0) {
        m_ResourceData = ::malloc(resourceSize);
        if (::GetFileVersionInfo(image, NULL, resourceSize, m_ResourceData)) {
            // Pick up the language ID used by this version resource.
            // Does not work with multi-language resources.
            UINT dataSize;
            LPVOID valuePointer;
            if (::VerQueryValue(m_ResourceData, _T("\\VarFileInfo\\Translation"), &valuePointer, &dataSize)) {
                // Must know the size of the scalar type used.
                // I bet there are more portable ways of doing this...
                __int16* lang = reinterpret_cast<__int16*>(valuePointer);
                m_LangID.Format("%04x%04x", lang[0], lang[1]);
            }
        }
    }
}

// Attribute helper

CString VersionInfo::GetValue(LPCTSTR key) const
{
    CString str;
    LPVOID valuePointer;
    UINT dataSize;

    ASSERT(m_ResourceData != NULL);

    str.Format("\\StringFileInfo\\%s\\%s", m_LangID, key);
    // We need a non-const pointer to the string data.
    ::VerQueryValue(m_ResourceData, str.GetBuffer(0), &valuePointer, &dataSize);
    str.ReleaseBuffer();
    str.Format("%s", reinterpret_cast<LPTSTR>(valuePointer));
    return str;
}
